@@ -1,115 +1,6 @@
 import casadi as cas
 import numpy as np
-
-
-def firefly_CLA_and_CDA_fuse_hybrid(
-        fuse_fineness_ratio,
-        fuse_boattail_angle,
-        fuse_TE_diameter,
-        fuse_length,
-        fuse_diameter,
-        alpha,
-        V,
-        mach,
-        rho,
-        mu,
-):
-    """
-    Estimated equiv. lift area and equiv. drag area of the Firefly fuselage, component buildup.
-    :param fuse_fineness_ratio: Fineness ratio of the fuselage nose (length / diameter)
-    :param fuse_boattail_angle: Boattail half-angle [deg]
-    :param fuse_TE_diameter: Diameter of the fuselage's base at the "trailing edge" [m]
-    :param fuse_length: Length of the fuselage [m]
-    :param fuse_diameter: Diameter of the fuselage [m]
-    :param alpha: Angle of attack [deg]
-    :param V: Airspeed [m/s]
-    :param mach: Mach number [unitless]
-    :param rho: Air density [kg/m^3]
-    :param mu: Dynamic viscosity of air [Pa*s]
-    :return: A tuple of (CLA, CDA) [m^2]
-    """
-    alpha_rad = alpha * np.pi / 180
-    sin_alpha = cas.sin(alpha_rad)
-    cos_alpha = cas.cos(alpha_rad)
-
-    """
-    Lift of a truncated fuselage, following slender body theory.
-    """
-    separation_location = 0.3  # 0 for separation at trailing edge, 1 for separation at start of boat-tail. Calibrate this.
-    diameter_at_separation = (1 - separation_location) * fuse_TE_diameter + separation_location * fuse_diameter
-
-    fuse_TE_area = np.pi / 4 * diameter_at_separation ** 2
-    CLA_slender_body = 2 * fuse_TE_area * alpha_rad  # Derived from FVA 6.6.5, Eq. 6.75
-
-    """
-    Crossflow force, following the method of Jorgensen, 1977: "Prediction of Static Aero. Chars. for Slender..."
-    """
-    V_crossflow = V * sin_alpha
-    Re_crossflow = rho * cas.fabs(V_crossflow) * fuse_diameter / mu
-    mach_crossflow = mach * cas.fabs(sin_alpha)
-    eta = 1  # TODO make this a function of overall fineness ratio
-    Cdn = 0.2  # Taken from suggestion for supercritical cylinders in Hoerner's Fluid Dynamic Drag, pg. 3-11
-    S_planform = fuse_diameter * fuse_length
-    CNA = eta * Cdn * S_planform * sin_alpha * cas.fabs(sin_alpha)
-    CLA_crossflow = CNA * cos_alpha
-    CDA_crossflow = CNA * sin_alpha
-
-    CLA = CLA_slender_body + CLA_crossflow
-
-    """
-    Zero-lift_force drag_force
-    Model derived from high-fidelity data at: C:\Projects\GitHub\firefly_aerodynamics\Design_Opt\studies\Circular Firefly Fuse CFD\Zero-Lift Drag
-    """
-    c = np.array([112.57153128951402720758778741583,
-                  -7.1720570832587240417410612280946,
-                  -0.01765596807595304351679033061373,
-                  0.0026135564778264172743071913629365,
-                  550.8775012129947299399645999074,
-                  3.3166868391027000129156476759817,
-                  11774.081980549422951298765838146,
-                  3073258.204571904614567756652832,
-                  0.0299,
-                  ])  # coefficients
-    fuse_Re = rho * cas.fabs(V) * fuse_length / mu
-    CDA_zero_lift = (
-            (
-                    c[0] * cas.exp(c[1] * fuse_fineness_ratio) +
-                    c[2] * fuse_boattail_angle +
-                    c[3] * fuse_boattail_angle ** 2 +
-                    c[4] * fuse_TE_diameter ** 2 +
-                    c[5]
-            ) / c[6] *
-            (fuse_Re / c[7]) ** (-1 / 7) *
-            (fuse_length * fuse_diameter) / c[8]
-    )
-
-    """
-    Assumes a ring-shaped wake projection into the Trefftz plane with a sinusoidal circulation distribution.
-    This results in a uniform grad(phi) within the ring in the Trefftz plane.
-    Derivation at C:\Projects\GitHub\firefly_aerodynamics\Gists and Ideas\Ring Wing Potential Flow
-    """
-    fuse_oswalds_efficiency = 0.5
-    CDiA_slender_body = CLA ** 2 / (
-            diameter_at_separation ** 2 * np.pi * fuse_oswalds_efficiency) / 2  # or equivalently, use the next line
-    # CDiA_slender_body = fuse_TE_area * alpha_rad ** 2 / 2
-
-    CDA = CDA_crossflow + CDiA_slender_body + CDA_zero_lift
-
-    return CLA, CDA
-
-
-def firefly_CLA_and_CDA_nominal_fuse_CFD(alpha):
-    """
-    Estimated equiv. lift area and equiv. drag area of the Firefly fuselage, fit to CFD data.
-    Calculated for fuse_fineness_ratio = 1.5, fuse_boattail_angle = 12, and fuse_TE_diameter = 20 mm.
-    Lift: R^2 = 0.9961
-    Drag: R^2 = 0.9876
-    :param alpha: angle of attack [deg]
-    :return: A tuple of (CLA, CDA) [m^2]
-    """
-    CLA = (0.4605 * alpha) / 1e4
-    CDA = (3.201 + 0.009754 * alpha ** 2) / 1e4
-    return CLA, CDA
+from aerosandbox.tools.casadi_tools import *
 
 
 def Cd_cylinder(
@@ -436,56 +327,189 @@ def Cd_wave_Korn(Cl, t_over_c, mach, sweep=0, kappa_A=0.95):
     return Cd_wave
 
 
+def induced_drag_ratio_from_ground_effect(
+        h_over_b  # type: float
+):
+    """
+    Gives the ratio of actual induced drag to free-flight induced drag experienced by a wing in ground effect.
+    Artificially smoothed below around h/b == 0.05 to retain differentiability and practicality.
+    Source: W. F. Phillips, D. F. Hunsaker, "Lifting-Line Predictions for Induced Drag and Lift in Ground Effect".
+        Using Equation 5 from the paper, which is modified from a model from Torenbeek:
+            Torenbeek, E. "Ground Effects", 1982.
+    :param h_over_b: (Height above ground) divided by (wingspan).
+    :return: Ratio of induced drag in ground effect to induced drag out of ground effect [unitless]
+    """
+    h_over_b = smoothmax(
+        h_over_b,
+        0,
+        hardness=1/0.03
+    )
+    return 1 - cas.exp(
+        -4.01 * (2 * h_over_b) ** 0.717
+    )
+
+
+def firefly_CLA_and_CDA_fuse_hybrid(  # TODO remove
+        fuse_fineness_ratio,
+        fuse_boattail_angle,
+        fuse_TE_diameter,
+        fuse_length,
+        fuse_diameter,
+        alpha,
+        V,
+        mach,
+        rho,
+        mu,
+):
+    """
+    Estimated equiv. lift area and equiv. drag area of the Firefly fuselage, component buildup.
+    :param fuse_fineness_ratio: Fineness ratio of the fuselage nose (length / diameter)
+    :param fuse_boattail_angle: Boattail half-angle [deg]
+    :param fuse_TE_diameter: Diameter of the fuselage's base at the "trailing edge" [m]
+    :param fuse_length: Length of the fuselage [m]
+    :param fuse_diameter: Diameter of the fuselage [m]
+    :param alpha: Angle of attack [deg]
+    :param V: Airspeed [m/s]
+    :param mach: Mach number [unitless]
+    :param rho: Air density [kg/m^3]
+    :param mu: Dynamic viscosity of air [Pa*s]
+    :return: A tuple of (CLA, CDA) [m^2]
+    """
+    alpha_rad = alpha * np.pi / 180
+    sin_alpha = cas.sin(alpha_rad)
+    cos_alpha = cas.cos(alpha_rad)
+
+    """
+    Lift of a truncated fuselage, following slender body theory.
+    """
+    separation_location = 0.3  # 0 for separation at trailing edge, 1 for separation at start of boat-tail. Calibrate this.
+    diameter_at_separation = (1 - separation_location) * fuse_TE_diameter + separation_location * fuse_diameter
+
+    fuse_TE_area = np.pi / 4 * diameter_at_separation ** 2
+    CLA_slender_body = 2 * fuse_TE_area * alpha_rad  # Derived from FVA 6.6.5, Eq. 6.75
+
+    """
+    Crossflow force, following the method of Jorgensen, 1977: "Prediction of Static Aero. Chars. for Slender..."
+    """
+    V_crossflow = V * sin_alpha
+    Re_crossflow = rho * cas.fabs(V_crossflow) * fuse_diameter / mu
+    mach_crossflow = mach * cas.fabs(sin_alpha)
+    eta = 1  # TODO make this a function of overall fineness ratio
+    Cdn = 0.2  # Taken from suggestion for supercritical cylinders in Hoerner's Fluid Dynamic Drag, pg. 3-11
+    S_planform = fuse_diameter * fuse_length
+    CNA = eta * Cdn * S_planform * sin_alpha * cas.fabs(sin_alpha)
+    CLA_crossflow = CNA * cos_alpha
+    CDA_crossflow = CNA * sin_alpha
+
+    CLA = CLA_slender_body + CLA_crossflow
+
+    """
+    Zero-lift_force drag_force
+    Model derived from high-fidelity data at: C:\Projects\GitHub\firefly_aerodynamics\Design_Opt\studies\Circular Firefly Fuse CFD\Zero-Lift Drag
+    """
+    c = np.array([112.57153128951402720758778741583,
+                  -7.1720570832587240417410612280946,
+                  -0.01765596807595304351679033061373,
+                  0.0026135564778264172743071913629365,
+                  550.8775012129947299399645999074,
+                  3.3166868391027000129156476759817,
+                  11774.081980549422951298765838146,
+                  3073258.204571904614567756652832,
+                  0.0299,
+                  ])  # coefficients
+    fuse_Re = rho * cas.fabs(V) * fuse_length / mu
+    CDA_zero_lift = (
+            (
+                    c[0] * cas.exp(c[1] * fuse_fineness_ratio) +
+                    c[2] * fuse_boattail_angle +
+                    c[3] * fuse_boattail_angle ** 2 +
+                    c[4] * fuse_TE_diameter ** 2 +
+                    c[5]
+            ) / c[6] *
+            (fuse_Re / c[7]) ** (-1 / 7) *
+            (fuse_length * fuse_diameter) / c[8]
+    )
+
+    """
+    Assumes a ring-shaped wake projection into the Trefftz plane with a sinusoidal circulation distribution.
+    This results in a uniform grad(phi) within the ring in the Trefftz plane.
+    Derivation at C:\Projects\GitHub\firefly_aerodynamics\Gists and Ideas\Ring Wing Potential Flow
+    """
+    fuse_oswalds_efficiency = 0.5
+    CDiA_slender_body = CLA ** 2 / (
+            diameter_at_separation ** 2 * np.pi * fuse_oswalds_efficiency) / 2  # or equivalently, use the next line
+    # CDiA_slender_body = fuse_TE_area * alpha_rad ** 2 / 2
+
+    CDA = CDA_crossflow + CDiA_slender_body + CDA_zero_lift
+
+    return CLA, CDA
+
+
+def firefly_CLA_and_CDA_nominal_fuse_CFD(alpha):  # TODO remove
+    """
+    Estimated equiv. lift area and equiv. drag area of the Firefly fuselage, fit to CFD data.
+    Calculated for fuse_fineness_ratio = 1.5, fuse_boattail_angle = 12, and fuse_TE_diameter = 20 mm.
+    Lift: R^2 = 0.9961
+    Drag: R^2 = 0.9876
+    :param alpha: angle of attack [deg]
+    :return: A tuple of (CLA, CDA) [m^2]
+    """
+    CLA = (0.4605 * alpha) / 1e4
+    CDA = (3.201 + 0.009754 * alpha ** 2) / 1e4
+    return CLA, CDA
+
+
 if __name__ == "__main__":
-    # Run some checks
-    import matplotlib.pyplot as plt
-    import matplotlib.style as style
-    import plotly.express as px
-    import plotly.graph_objects as go
-    import dash
-    style.use("seaborn")
-
-
-    # # E216 checks
-    alpha_inputs = np.linspace(-6, 12, 200)
-    Re_inputs = np.logspace(4, 6, 200)
-    alphas = []
-    Res = []
-    CLs = []
-    CDs = []
-    for alpha in alpha_inputs:
-        for Re in Re_inputs:
-            alphas.append(alpha)
-            Res.append(Re)
-            CLs.append(Cl_e216(alpha, Re))
-            CDs.append(Cd_profile_e216(alpha, Re))
-    px.scatter_3d(
-        x=alphas,
-        y=Res,
-        z=CLs,
-        size=np.ones_like(alphas),
-        color=CLs,
-        log_y=True,
-        labels={"x": "alphas", "y": "Re", "z": "CL"}
-    ).show()
-    px.scatter_3d(
-        x=alphas,
-        y=Res,
-        z=CDs,
-        size=np.ones_like(alphas),
-        color=CDs,
-        log_y=True,
-        labels={"x": "alphas", "y": "Re", "z": "CD"}
-    ).show()
-    px.scatter_3d(
-        x=alphas,
-        y=Res,
-        z=np.array(CLs) / np.array(CDs),
-        size=np.ones_like(alphas),
-        color=np.array(CLs) / np.array(CDs),
-        log_y=True,
-        labels={"x": "alphas", "y": "Re", "z": "CL/CD"}
-    ).show()
+    pass
+    # # Run some checks
+    # import matplotlib.pyplot as plt
+    # import matplotlib.style as style
+    # import plotly.express as px
+    # import plotly.graph_objects as go
+    # import dash
+    #
+    # style.use("seaborn")
+    #
+    # # # E216 checks
+    # alpha_inputs = np.linspace(-6, 12, 200)
+    # Re_inputs = np.logspace(4, 6, 200)
+    # alphas = []
+    # Res = []
+    # CLs = []
+    # CDs = []
+    # for alpha in alpha_inputs:
+    #     for Re in Re_inputs:
+    #         alphas.append(alpha)
+    #         Res.append(Re)
+    #         CLs.append(Cl_e216(alpha, Re))
+    #         CDs.append(Cd_profile_e216(alpha, Re))
+    # px.scatter_3d(
+    #     x=alphas,
+    #     y=Res,
+    #     z=CLs,
+    #     size=np.ones_like(alphas),
+    #     color=CLs,
+    #     log_y=True,
+    #     labels={"x": "alphas", "y": "Re", "z": "CL"}
+    # ).show()
+    # px.scatter_3d(
+    #     x=alphas,
+    #     y=Res,
+    #     z=CDs,
+    #     size=np.ones_like(alphas),
+    #     color=CDs,
+    #     log_y=True,
+    #     labels={"x": "alphas", "y": "Re", "z": "CD"}
+    # ).show()
+    # px.scatter_3d(
+    #     x=alphas,
+    #     y=Res,
+    #     z=np.array(CLs) / np.array(CDs),
+    #     size=np.ones_like(alphas),
+    #     color=np.array(CLs) / np.array(CDs),
+    #     log_y=True,
+    #     labels={"x": "alphas", "y": "Re", "z": "CL/CD"}
+    # ).show()
     #
     # # # rae2822 checks
     # alpha_inputs = np.linspace(-6, 12)
@@ -594,15 +618,15 @@ if __name__ == "__main__":
     #     range_z=(0, 200e-4)
     # ).show()
 
-    # # Cylinder Drag Check
-    Res = np.logspace(-1, 8, 300)
-    CDs = Cd_cylinder(Res)
-    CDs_s = Cd_cylinder(Res, True)
-
-    plt.loglog(Res, CDs, label="Full Model")
-    plt.loglog(Res, CDs_s, label="Subcrit. Only Model")
-    plt.xlabel("Re")
-    plt.ylabel("CD")
-    plt.title("Cylinder Drag Checking")
-    plt.legend()
-    plt.show()
\ No newline at end of file
+    # # # Cylinder Drag Check
+    # Res = np.logspace(-1, 8, 300)
+    # CDs = Cd_cylinder(Res)
+    # CDs_s = Cd_cylinder(Res, True)
+    #
+    # plt.loglog(Res, CDs, label="Full Model")
+    # plt.loglog(Res, CDs_s, label="Subcrit. Only Model")
+    # plt.xlabel("Re")
+    # plt.ylabel("CD")
+    # plt.title("Cylinder Drag Checking")
+    # plt.legend()
+    # plt.show()
